#
# dialog.tcl - generic dialog creation command
# 
# Copyright (C) 2005  Toni Ronkko
# 
# Peach Gardens is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Peach Gardens is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Aug 23, 2004  Toni Ronkko <toni at users.playstation2-linux.com>
# 
# $Id: dialog.tcl,v 1.2 2005/01/09 20:52:12 tronkko Exp $
#
# $Log: dialog.tcl,v $
# Revision 1.2  2005/01/09 20:52:12  tronkko
# ensure that window is visible before grabbing input
#
# Revision 1.1  2004/11/28 11:08:14  tronkko
# Initial revision
#
#
#


#
# Create a simple dialog window with upper and lower parts.  Returns a
# handle refering to the window.  Pass the handle returned to dialog_info
# or dialog_controls function to get handle to upper or lower half of the
# dialog window created.
# 
proc dialog_create { name {win "auto"} } {
  # create name for the window if the caller did not supply one
  if { $win == "auto" } {
    set count 0
    set win ".dialog[incr count]"
    while {[winfo exists $win]} {
      set win ".dialog[incr count]"
    }
  }

  # create new toplevel window
  toplevel $win -class $name
  frame $win.f
  pack $win.f -padx 4 -pady 4 -fill both -expand yes

  # create components
  frame [dialog_info $win] 
  frame [dialog_controls $win]
  
  pack [dialog_controls $win] -fill x -side bottom -padx 4 -pady 4
  pack [dialog_info $win] -expand yes -fill both -side top

  # Set window class.  The class is useful for defining resources.  With 
  # resources the user can override some colors, for example.
  wm title $win $name

  # Tell window manager that the dialog window belongs to the very same
  # group as the main window.  This helps window managers to keep the
  # dialog on the top of the main window.
  wm group $win .

  # set minimum size as soon as the window is visible
  after idle "dialog_set_minsize $win"
  return "$win"
}


#
# Get name of the upper part of the window created by dialog_create.
#
proc dialog_info { win } {
  return "$win.f.info"
}


#
# Get name of lower part of the window created by dialog_create.
#
proc dialog_controls { win } {
  return "$win.f.controls"
}


#
# Raise modal dialog to top when clicked.  Sometimes small dialog windows
# are buried under other windows, and user may have trouble understanding
# why the program has stopped responding if he cannot see the dialog.
#
bind modalDialog <ButtonPress> {
  # give user an audible signal: no, you cannot touch that
  bell
  
  # raise dialog to the top
  wm deiconify %W
  raise %W
  focus %W
}


#
# Wait until a modal dialog is closed or a variable is set.
#
proc dialog_wait { win {varname ""} } {
  
  # bind the modalDialog event to the window
  if { [lsearch [bindtags $win] modalDialog] < 0 } {
    bindtags $win [linsert [bindtags $win] 0 modalDialog]
  }

  # Save previous event grab.  This is to cope with multiple modal dialogs
  # on the screen at the very same time.  This can happen if user tries to
  # close the main window while on a modal dialog, and the main window then
  # posts a question on another modal dialog.
  set prev [grab current]
  if { $prev != "" } {
    if { [winfo toplevel $prev] != $prev } {
      set prev ""
    }
  }

  # If the dialog window has been just created, it may not be visible
  # on screen just yet.  Update idle tasks to ensure that the window is
  # visible.
  update idletasks

  # grab input and wait for release
  grab set $win
  if { $varname == "" } { tkwait window $win }
  if { $varname != "" } { tkwait variable $varname }
  grab release $win

  # return previous event grab
  if { $prev != "" } {
    if { [winfo exists $prev] != 0  &&  [winfo ismapped $prev] != 0 } {
      grab set $prev
    }
  }
}


#
# Create modal dialog, and wait until user closes it.
# Bitmap is usually one of: error, info, questhead, question or warning.
#
proc modal_dialog { title text bitmap { string  "OK" } { def 0 } } {
  global _modal_dialog
  
  # create modal dialog
  set win [dialog_create $title]
  set info [dialog_info $win]
  set cntls [dialog_controls $win]

  # create text and icon
  label $info.icon -bitmap $bitmap
  pack $info.icon -side left -padx 8 -pady 8
  label $info.mesg -wraplength 4i -text $text
  pack $info.mesg -side right -expand yes -fill both -padx 8 -pady 8

  # create the buttons at the bottom of the dialog
  frame $cntls.f
  pack $cntls.f -expand yes

  set count 0
  foreach i $string {
    set button $cntls.f.$count
    button $button -text $i \
        -command "set _modal_dialog($win) $count ; destroy $win"
    pack $button -side left -padx 4 
    bind $button <Return> "%W invoke"

    # Aseta focus oletuskenttn
    if { $count == $def } { focus $button }
    
    incr count
  }

  # set default button
  wm protocol $win WM_DELETE_WINDOW \
      "set _modal_dialog($win) $def ; destroy $win"
  bind $win <Escape> "set _modal_dialog($win) -1 ; destroy $win"

  # move dialog under the mouse cursor
  set x [expr [winfo pointerx $win] - [winfo reqwidth $win]/2]
  set y [expr [winfo pointery $win] - [winfo reqheight $win]/2]
  if { $x < 0 } { set x 0 }
  if { $y < 0 } { set y 0 }
  wm geometry $win "+$x+$y"

  # show dialog, and wait for its closure
  dialog_wait $win
  return $_modal_dialog($win)
}


#
# Restrict a window from being minimized from the current size.
#
proc dialog_set_minsize { win } {
  update idletasks
  set menu [$win cget -menu]
  if { $menu != "" } {
    set menu_height [winfo reqheight $menu]
  } else {
    set menu_height 0
  }
  
  if { [wm minsize $win] == "1 1" } {
    wm minsize $win [winfo reqwidth $win] \
        [expr [winfo reqheight $win] + $menu_height]
  }
}
