#
# pane.tcl - paned window
#
# This very module is based on a code taken from book "Effective Tcl/Tk
# Programming" by Mark Harrison and Michael McLennan.  The code has been
# further modified by Toni Ronkko.
#
#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# 
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================
#
#
# Sep 5, 2004  Toni Ronkko <toni at users.playstation2-linux.com>
# 
# $Id: pane.tcl,v 1.1 2004/11/28 11:08:43 tronkko Exp $
#
# $Log: pane.tcl,v $
# Revision 1.1  2004/11/28 11:08:43  tronkko
# Initial revision
#
#
#


# default cursor
option add *Pane.sash.cursor sb_h_double_arrow widgetDefault
option add *Pane.grip.cursor sb_h_double_arrow widgetDefault

# color of background
set pw_color ""


#
# Create horizontally paned window with a sash to resize the left and right
# portion.
#
proc create_pane {win width height} {
  global pw_color
  
  # frame containing the divided parts
  frame $win -class Pane -width $width -height $height

  # left and right panes
  frame $win.left
  place $win.left -relx 0.0 -rely 0.5 \
      -anchor w -relwidth 0.0 -relheight 1.0
  frame $win.right
  place $win.right -relx 1.0 -rely 0.5 \
      -anchor e -relwidth 1.0 -relheight 1.0

  # divider between frames
  frame $win.sash -width 4 -borderwidth 2 -relief raised
  place $win.sash -relx 0.5 -rely 0.5 -relheight 1.0 -anchor c

  # remember the original background color of the sash
  set pw_color [$win.sash cget -bg]

  # grip
  frame $win.grip -width 10 -height 10 -borderwidth 2 -relief raised
  place $win.grip -relx 0.5 -rely 0.85 -anchor c

  # hints
  bind $win.grip <Enter> \
      "status {Hold down left mouse button and move mouse to resize}"
  bind $win.grip <Leave> \
      "status {}"

  # set default width
  divide_pane $win 0.5

  # grabbing sash
  bind $win.sash <ButtonPress-1>   "grab_pane $win"
  bind $win.sash <B1-Motion>       "drag_pane $win %X"
  bind $win.sash <ButtonRelease-1> "drop_pane $win %X"
  bind $win.grip <ButtonPress-1>   "grab_pane $win"
  bind $win.grip <B1-Motion>       "drag_pane $win %X"
  bind $win.grip <ButtonRelease-1> "drop_pane $win %X"
  bind $win <Motion> ""

  return $win
}


#
# Start dragging the sash.  The function is invoked by the user pressing
# the mouse button down above the sash.
# 
proc grab_pane {win} {
  # turn sash into black one-pixel wide line, hide grip
  $win.sash configure -relief flat -width 1 -borderwidth 0 -bg black

  # Hide the grip outside the screen boundaries to make the drag look
  # better.  Moving the grip with the sash is not very pretty.  Note that
  # we cannot forget the grip here, as we really need it on screen to do
  # grabbing.
  place $win.grip -relx -1
}


#
# Drag sash to resize windows.
#
proc drag_pane {win x} {
  # compute fraction
  set realx [expr $x - [winfo rootx $win]]
  set xmax  [winfo width $win]
  set frac [expr double($realx)/$xmax]

  # restrict the width of parts
  if {$frac < 0.0} {
    set frac 0.0
  }
  if {$frac > 1.0} {
    set frac 1.0
  }
  place $win.sash -relx $frac
  return $frac
}


#
# End drag.
#
proc drop_pane {win x} {
  global pw_color

  # resize left and right windows
  set frac [drag_pane $win $x]
  divide_pane $win $frac

  # restore sash
  $win.sash configure -width 4 -borderwidth 2 -relief raised -bg $pw_color
}

#
# Divide window horizontally.
#
proc divide_pane {win frac} {
  place $win.sash -relx $frac
  place $win.grip -relx $frac

  place $win.left -relwidth $frac
  place $win.right -relwidth [expr 1-$frac]
}


