#
# workdlg.tcl - dialogs for creating and editing workspaces
# 
# Copyright (C) 2005  Toni Ronkko
#
# Peach Gardens is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# Peach Gardens is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Nov 2004, Toni Ronkko <toni at users.playstation2-linux.com>
#
# $Id: workdlg.tcl,v 1.3 2005/01/16 23:36:08 tronkko Exp $
#
# $Log: workdlg.tcl,v $
# Revision 1.3  2005/01/16 23:36:08  tronkko
# default workspace name
#
# Revision 1.2  2005/01/09 21:00:30  tronkko
# updated copyright
#
# Revision 1.1  2004/12/09 22:15:03  tronkko
# Initial revision
#
#
#


# default size of new workspaces
set workspace_dialog_size 0

# size of custom workspace
set workspace_dialog_width  160
set workspace_dialog_height 160

# name of workspace
set workspace_dialog_name ""

# result of workspace dialog
set workspace_dialog_result ""


#
# Show a dialog window asking for workspace properties, and create a
# workspace accordingly.  Note that the very same window is used for both
# creating palettes and maps.  (See README.2 file for more information
# about the terms.)  Hence, avoid using words "workspace", "map" and
# "palette" in the dialog and in the error messages to avoid confusion.
#
# Returns the index of the new workspace, or -1 if user aborted the
# operation.
#
proc create_workspace_dialog { title type {suggest ""} } {
  global workspace_dialog_result
  global workspace_dialog_width
  global workspace_dialog_height
  global workspace_dialog_name
  global nworkspaces
  global workspace_name
  global workspace_dialog_size
  set result -1
  
  # create window with default values in the widgets
  set workspace_dialog_name $suggest
  set win [workspace_dialog $title]

  # insert buttons to lower part of the dialog
  set up [dialog_info $win]
  set lo [dialog_controls $win]

  # wait for the user to press either ok or cancel
  set done 0
  while { ! $done } {
    # wait for the user to make a decision
    set workspace_dialog_result ""
    dialog_wait $win workspace_dialog_result

    # user wants to create a workspace
    if { $workspace_dialog_result == "ok" } {

      # fetch workspace name from dialog
      set name [get_workspace_name $win]
      if { $name != "" } {
        set ok 1
      } else {
        # invalid name
        set ok 0
      }

      # ensure that name does not yet exist
      if { $ok } {
        for {set i 0} {$i < $nworkspaces} {incr i} {
          if { $workspace_name($i) == $name } {
            set ok 0
            modal_dialog "Error" \
"Name '$name' is already taken.  Please choose another name." error
            focus $up.frame.name
          }
        }
      }

      # get size into variables w and h
      if { $ok } {
        set r [get_workspace_size $win]
        if { $r != "" } {
          set w [lindex $r 0]
          set h [lindex $r 1]
        } else {
          # invalid size
          set ok 0
        }
      }

      # create workspace
      if { $ok } {
        set i [new_workspace $type $workspace_dialog_name $w $h]

        # exit from event loop
        set done 1
        set result $i        
      }
      
    } else {
      status "Cancel"
      set result -1
      set done 1
    }
  }

  # close workspace dialog window
  destroy $win

  return $result
}


#
# Edit name or resize workspace.  Returns true if the workspace was changed
# to tell the caller to update display.
#
proc edit_workspace_dialog { title cur } {
  global workspace_dialog_result
  global workspace_dialog_size
  global workspace_dialog_width
  global workspace_dialog_height
  global workspace_dialog_name
  global workspace_name
  global workspace_width
  global workspace_height
  global nworkspaces
  set result 0
  
  # specify initial size in dialog
  set w $workspace_width($cur)
  set h $workspace_height($cur)
  if { $w == 10  &&  $h == 20 } {
    set workspace_dialog_size 0
  } elseif { $w == 40  &&  $h == 40 } {
    set workspace_dialog_size 1
  } elseif { $w == 80  &&  $h == 80 } {
    set workspace_dialog_size 2
  } else {
    # custom size
    set workspace_dialog_size 3
  }

  # always set custom size to the current one
  set workspace_dialog_width  $w
  set workspace_dialog_height $h

  # set default name
  set workspace_dialog_name $workspace_name($cur)
  
  # create workspace dialog
  set win [workspace_dialog $title]

  # insert buttons to lower part of the dialog
  set up [dialog_info $win]
  set lo [dialog_controls $win]

  # wait for the user to press either ok or cancel
  set done 0
  while { ! $done } {
    # wait for the user to make a decision
    set workspace_dialog_result ""
    dialog_wait $win workspace_dialog_result

    # user wants to create a workspace
    if { $workspace_dialog_result == "ok" } {

      # fetch workspace name from dialog
      set name [get_workspace_name $win]
      if { $name != "" } {
        set ok 1
      } else {
        # invalid name
        set ok 0
      }

      # ensure that the name is not used by any other workspace
      if { $ok } {
        for {set i 0} {$i < $nworkspaces} {incr i} {
          if { $i != $cur  &&  $workspace_name($i) == $name } {
            set ok 0
            modal_dialog "Error" \
"Name '$name' is already taken.  Please choose another name." error
            focus $up.frame.name
          }
        }
      }

      # get size into variables w and h
      if { $ok } {
        set r [get_workspace_size $win]
        if { $r != "" } {
          set w [lindex $r 0]
          set h [lindex $r 1]
        } else {
          # invalid size
          set ok 0
        }
      }

      # confirm lose of chips
      if {$ok && ($w<$workspace_width($cur) || $h<$workspace_height($cur))} {
        set button [modal_dialog \
            "Continue with Resize?" \
            "Some chips may be lost due to the new size, and the operation\
cannot be undone.  Do you really want to continue?" \
question {Yes No} 0]
        if { $button == 0 } {
          # yes
          set ok 1
        } else {
          # no
          set ok 0
        }
      }

      # modify workspace
      if { $ok } {       
        # save name
        set workspace_name($cur) $name

        # resize if needed
        resize_workspace $cur $w $h
        
        # exit from loop
        set result 1
        set done 1
      }
      
    } else {
      status "Cancel"
      set result 0
      set done 1
    }
  }
  
  # close workspace dialog
  destroy $win
  return $result
}


#
# Create dialog showing workspace properties.  The initial values of the
# dialog are taken from variables workspace_dialog_size,
# workspace_dialog_width, workspace_dialog_height and
# workspace_dialog_name.  Set the variables accordingly to edit an existing
# workspace.
#
proc workspace_dialog { title } {
  global workspace_dialog_name
  global workspace_dialog_width
  global workspace_dialog_height
  global have_validate
  
  # create dialog window
  set win [dialog_create Workspace]
  set up [dialog_info $win]
  set lo [dialog_controls $win]

  # set title of the dialog
  wm title $win $title

  # frame containing elements of upper part
  frame $up.frame
  
  # create upper half of the dialog
  label $up.frame.name_text -text "Name:"
  if { $have_validate } {
    entry $up.frame.name \
        -validatecommand "validate_workspace_name %P" -validate key
  } else {
    # Tk 8.0
    entry $up.frame.name
  }
  $up.frame.name insert 0 $workspace_dialog_name
  
  # size and radio buttons
  label $up.frame.size_text -text "Size:"
  frame $up.frame.size
  frame $up.frame.size.1
  radiobutton $up.frame.size.1.option -text "10x20" \
      -variable workspace_dialog_size -value 0
  frame $up.frame.size.2
  radiobutton $up.frame.size.2.option -text "40x40" \
      -variable workspace_dialog_size -value 1
  frame $up.frame.size.3
  radiobutton $up.frame.size.3.option -text "80x80" \
      -variable workspace_dialog_size -value 2
  frame $up.frame.size.4
  radiobutton $up.frame.size.4.option -text "Other:" \
      -variable workspace_dialog_size -value 3
  if { $have_validate } {
    entry $up.frame.size.4.width -width 3 \
        -validatecommand "validate_workspace_size %P" -validate key
    entry $up.frame.size.4.height -width 3 \
        -validatecommand "validate_workspace_size %P" -validate key
  } else {
    # Tk 8.0
    entry $up.frame.size.4.width -width 3
    entry $up.frame.size.4.height -width 3
  }
  label $up.frame.size.4.x -text "x"

  # initial width and height
  $up.frame.size.4.width insert 0 $workspace_dialog_width
  $up.frame.size.4.height insert 0 $workspace_dialog_height

  # button row
  frame $lo.buttons
  button $lo.buttons.ok -text "OK" -default active \
      -command "set workspace_dialog_result ok"
  button $lo.buttons.cancel -text "Cancel" \
      -command "set workspace_dialog_result cancel"

  # pack radio buttons
  pack $up.frame.size.1.option -side left
  pack $up.frame.size.2.option -side left
  pack $up.frame.size.3.option -side left
  pack $up.frame.size.4.option -side left
  pack $up.frame.size.4.width  -side left
  pack $up.frame.size.4.x      -side left
  pack $up.frame.size.4.height -side left
  
  # place radio buttons inside their fram
  pack $up.frame.size.1 -side top -anchor w
  pack $up.frame.size.2 -side top -anchor w
  pack $up.frame.size.3 -side top -anchor w
  pack $up.frame.size.4 -side top -anchor w
  
  # lay main elements in a grid
  grid $up.frame.name_text -row 0 -column 0 -sticky nw
  grid $up.frame.name      -row 0 -column 1 -sticky nwe
  grid $up.frame.size_text -row 1 -column 0 -sticky nw
  grid $up.frame.size      -row 1 -column 1 -sticky nw

  # frame containing upper part
  pack $up.frame -padx 8 -pady 8 -fill x -expand yes

  # pack buttons
  pack $lo.buttons.ok -side left -padx 4
  pack $lo.buttons.cancel -side left -padx 4
  pack $lo.buttons

  # start editing name
  focus $up.frame.name

  # bind events
  bind $win <Return> "set workspace_dialog_result ok"
  bind $win <Escape> "set workspace_dialog_result cancel"
  wm protocol $win WM_DELETE_WINDOW \
      "set workspace_dialog_result cancel"

  # editing width or height automatically selects other size
  bind $up.frame.size.4.width  <KeyPress> "workspace_activate_custom %A"
  bind $up.frame.size.4.height <KeyPress> "workspace_activate_custom %A"

  # return window name to caller
  return $win
}


#
# Automatically select custom field size when either the width or the
# height field is edited.
#
proc workspace_activate_custom { key } {
  global workspace_dialog_size

  if { $key != "\t"  &&  $key != "" } {
    set workspace_dialog_size 3
  }
}


#
# Get name of workspace from dialog.  Returns empty string if the name
# is not valid, and posts error dialogs to tell the user about it.
# However, does not check if the workspace name is already taken.
#
proc get_workspace_name { win } {
  global workspace_dialog_name
  
  # name of upper dialog part
  set up [dialog_info $win]

  # get name from edit control
  set workspace_dialog_name [$up.frame.name get]
  
  # workspace must be named
  set ok 1
  if { $workspace_dialog_name == "" } {
    modal_dialog "Error" "Missing name" error
    focus $up.frame.name
    set ok 0
  }
  if { $ok  &&  ! [validate_workspace_name $workspace_dialog_name] } {
    modal_dialog "Error" "Invalid name" error
    focus $up.frame.name
    set ok 0
  }

  # pass the name to caller
  if { $ok } {
    return $workspace_dialog_name
  } else {
    return ""
  }
}


#
# Get workspace width and height from dialog win.  Returns empty list
# and posts error dialogs if dimensions are not right.
#
proc get_workspace_size { win } {
  global workspace_dialog_size
  global workspace_dialog_width
  global workspace_dialog_height
  
  # name of upper dialog part
  set up [dialog_info $win]

  set ok 1
  if { $workspace_dialog_size == 0 } {
    set w 10
    set h 20
  } elseif { $workspace_dialog_size == 1 } {
    set w 40
    set h 40
  } elseif { $workspace_dialog_size == 2 } {
    set w 80
    set h 80
  } elseif { $workspace_dialog_size == 3 } {
    # custom size
    set w [$up.frame.size.4.width  get]
    set h [$up.frame.size.4.height get]

    # ensure that size is a number
    if { ! [validate_workspace_size $w] } {
      modal_dialog "Error" "Invalid number: $w" error
      focus $up.frame.size.4.width
      set ok 0
    }
    if { $ok  &&  ! [validate_workspace_size $h] } {
      modal_dialog "Error" "Invalid number: $h" error
      focus $up.frame.size.4.height
      set ok 0
    }
    
    # refuse to create a small workspace
    if { $ok  &&  $w < 3 } {
      modal_dialog "Error" "Must be at least 3 chips wide" error
      focus $up.frame.size.4.width
      set ok 0
    }
    if { $ok  &&  $h < 3 } {
      modal_dialog "Error" "Must be at least 3 chips tall" error
      focus $up.frame.size.4.height
      set ok 0
    }
    
    # remember size for next time
    if { $ok } {
      set workspace_dialog_width  $w
      set workspace_dialog_height $h
    }
  } else {
    modal_dialog "Error" "Invalid size: $workspace_dialog_size" error
    set ok 0
  }

  # pass dimensions to caller
  if { $ok } {
    return [list $w $h]
  } else {
    return ""
  }
}


#
# Check if argument w is valid size specification for the workspace
# input dialog.
#
proc validate_workspace_size { w } {
  # must be number
  set ok 1
  if { ! [regexp {^[1-9][0-9]*$} $w]  &&  $w != "" } {
    set ok 0
  }

  # must be of suitable size
  if { $ok } {
    if { ($w < 0  ||  $w > 999)  &&  $w != "" } {
      set ok 0
    }
  }

  # tell user about the problem
  if { ! $ok } { bell }
  return $ok
}


#
# Check if argument is valid to be used as a workspace name.
#
proc validate_workspace_name { name } {
  set ok [regexp {^[A-Za-z_][A-Za-z_0-9]*$} $name]
  if { ! $ok } { bell }
  return $ok
}
